/***********************************************************************
*
*  FILE        : tb_rx65n_main.cpp
*  DATE        : 2019-02-26
*  DESCRIPTION : Main Program
*
*  NOTE:THIS IS A TYPICAL EXAMPLE.
*
***********************************************************************/
#include "RenesasRX.h"
#include "tbrx65ndef.h"
#include "stdio.h"

#define MAKEWORD(l, h) ((uint8_t)(l) | (uint16_t)((uint8_t)(h) << 8))
#define HIBYTE(w) ((uint8_t)(((uint16_t)(w)) >> 8))
#define LOBYTE(w) ((uint8_t)(w))

extern volatile st_dtc_data_t dtc_transferdata_vector39; /* RSPI SEND */
extern volatile st_dtc_data_t dtc_transferdata_vector38; /* RSPI RECV */

volatile int32_t RSPI0_txint_count;
volatile int32_t RSPI0_txdtce_chk_0;
volatile int32_t RSPI0_txdtce_chk[10];
volatile int32_t RSPI0_txdtc_cra_chk_0;
volatile int32_t RSPI0_txdtc_cra_chk[10];

volatile int32_t RSPI0_rxint_count;
volatile int32_t RSPI0_rxdtce_chk_0;
volatile int32_t RSPI0_rxdtce_chk[10];
volatile int32_t RSPI0_rxdtc_cra_chk_0;
volatile int32_t RSPI0_rxdtc_cra_chk[10];

volatile bool RSPI0_idle;

#define RSPI0_DATA_LEN 3
uint32_t RSPI0_send_buff[RSPI0_DATA_LEN] = { MAKEWORD('B', 'A'), MAKEWORD('D', 'C'), MAKEWORD('F', 'E') };
uint32_t RSPI0_recv_buff[RSPI0_DATA_LEN] = { MAKEWORD('-', '-'), MAKEWORD('-', '-'), MAKEWORD('-', '-') };
uint32_t RSPI0_data_len = RSPI0_DATA_LEN;

int main(void)
{
    LED0 = LED_OFF;
    LED1 = LED_OFF;

    R_Config_DTC_RSPI_RECV_Start();
    R_Config_DTC_RSPI_SEND_Start();
    R_Config_RSPI0_Start();

    RSPI0_idle = false;

    RSPI0_txint_count = 0;
    RSPI0_txdtce_chk_0 = ICU.DTCER[39].BIT.DTCE;
    RSPI0_txdtc_cra_chk_0 = (dtc_transferdata_vector39.cra_crb >> 16);

    RSPI0_rxint_count = 0;
    RSPI0_rxdtce_chk_0 = ICU.DTCER[38].BIT.DTCE;
    RSPI0_rxdtc_cra_chk_0 = (dtc_transferdata_vector38.cra_crb >> 16);

    R_Config_RSPI0_Send_Receive( RSPI0_send_buff, RSPI0_data_len, RSPI0_recv_buff );

    while ( RSPI0_idle == false );

    R_Config_SCI6_Start();

    Printf( SCI6, "Loopback = \"%c%c\" \"%c%c\" \"%c%c\" --> \"%c%c\" \"%c%c\" \"%c%c\"\r\nRSPI+SEND(DTC)+RECV(DTC),Master,SPDR=32bit,Data=16bit\r\n", 
    HIBYTE(RSPI0_send_buff[0]), LOBYTE(RSPI0_send_buff[0]), HIBYTE(RSPI0_send_buff[1]), LOBYTE(RSPI0_send_buff[1]), HIBYTE(RSPI0_send_buff[2]), LOBYTE(RSPI0_send_buff[2]), 
    HIBYTE(RSPI0_recv_buff[0]), LOBYTE(RSPI0_recv_buff[0]), HIBYTE(RSPI0_recv_buff[1]), LOBYTE(RSPI0_recv_buff[1]), HIBYTE(RSPI0_recv_buff[2]), LOBYTE(RSPI0_recv_buff[2])  );

    if ( RSPI0_txint_count <= 1 )
    {
        Printf( SCI6, "SEND:\r\nINT count = %d\r\nDTCE = %d(init), %d(1st INT)\r\nCRA  = %d(init), %d(1st INT)\r\n", 
        RSPI0_txint_count, 
        RSPI0_txdtce_chk_0, RSPI0_txdtce_chk[0],
        RSPI0_txdtc_cra_chk_0, RSPI0_txdtc_cra_chk[0] );
    }
    else
    {
        Printf( SCI6, "SEND:\r\nINT count = %d\r\nDTCE = %d(init), %d(1st INT), %d(2nd INT)\r\nCRA  = %d(init), %d(1st INT), %d(2nd INT)\r\n", 
        RSPI0_txint_count, 
        RSPI0_txdtce_chk_0, RSPI0_txdtce_chk[0], RSPI0_txdtce_chk[1],
        RSPI0_txdtc_cra_chk_0, RSPI0_txdtc_cra_chk[0], RSPI0_txdtc_cra_chk[1] );
    }

    if ( RSPI0_rxint_count <= 1 )
    {
        Printf( SCI6, "RECV:\r\nINT count = %d\r\nDTCE = %d(init), %d(1st INT)\r\nCRA  = %d(init), %d(1st INT)\r\n", 
        RSPI0_rxint_count, 
        RSPI0_rxdtce_chk_0, RSPI0_rxdtce_chk[0],
        RSPI0_rxdtc_cra_chk_0, RSPI0_rxdtc_cra_chk[0] );
    }
    else
    {
        Printf( SCI6, "RECV:\r\nINT count = %d\r\nDTCE = %d(init), %d(1st INT), %d(2nd INT)\r\nCRA  = %d(init), %d(1st INT), %d(2nd INT)\r\n", 
        RSPI0_rxint_count, 
        RSPI0_rxdtce_chk_0, RSPI0_rxdtce_chk[0], RSPI0_rxdtce_chk[1],
        RSPI0_rxdtc_cra_chk_0, RSPI0_rxdtc_cra_chk[0], RSPI0_rxdtc_cra_chk[1] );
    }

    while (1)
    {
        ;
    }
}
